/** @file ex026.c Demo-Programm fuer Schreiben einer Binaer-Datei,
    portabel mit he-file Modul.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#include <stdio.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_PROCESS_H
#include <process.h>
#endif

#include "he-file.h"


/** Koordinaten fuer einen Punkt.
*/
typedef struct {
    double    x;  /**< x-Koordinate. */
    double    y;  /**< y-Koordinate. */
} PUNKT;


/** Achsenparalleles Rechteck.
*/
typedef struct {
    PUNKT lu; /**< Eckpunkt links unten. */
    PUNKT ro; /**< Eckpunkt rechts oben. */
} RECHTECK;


/** Rechteck-Daten.
*/
const RECHTECK re = { { 1.0, 2.0 }, { 3.0, 4.0} };


/** Dateiname der zu schreibenden Datei.
*/
#if HE_ON_WINDOWS
static const char filename[] = { "C:\\Temp\\datafile.dat" };
#else
static const char filename[] = { "datafile.dat" };
#endif


/** Hauptprogramm.
    @return 0 bei Erfolg, alle anderen Werte bedeuten Fehler.
*/
int
main(void)
{
    FILE      *outf    = NULL;            /* Datei-Zeiger */
    int        exc     = EXIT_FAILURE;    /* Exit-Code des Programmes. */
    size_t     res;                       /* Ergebnis von fwrite() */

    outf = hsm_et_file_open(filename, "wb", NULL);
    if (NULL != outf) {

        /* Erfolg im Rueckgabecode vermerken */
        exc = EXIT_SUCCESS;

        /* Wenn weniger als ein Rechteck geschrieben wurde, trat Fehler auf */
        res = fwrite(&re, sizeof(RECHTECK), 1, outf);
        if (1 > res) {
          exc = EXIT_FAILURE;
        }

        /* Falls Schliessen der Datei fehlschlaegt, Misserfolg */
        if (0 >= hsm_et_file_close(outf, filename, NULL)) {
          exc = EXIT_FAILURE;
        }
        outf = NULL;
    }
    exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 expandtab : */
