/** @file ex029.c Demo-Programm fuer Lesen einer Binaer-Datei,
    portabel mit he-file Modul.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include "he-conf.h"

#include <stdio.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_PROCESS_H
#include <process.h>
#endif

#include "he-file.h"


/** Koordinaten fuer einen Punkt.
*/
typedef struct {
    double    x;  /**< x-Koordinate. */
    double    y;  /**< y-Koordinate. */
} PUNKT;


/** Achsenparalleles Rechteck.
*/
typedef struct {
    PUNKT lu; /**< Eckpunkt links unten. */
    PUNKT ro; /**< Eckpunkt rechts oben. */
} RECHTECK;


/** Dateiname der zu lesenden Datei.
*/
#if HE_ON_WINDOWS
static const char filename[] = { "C:\\Temp\\datafile.dat" };
#else
static const char filename[] = { "datafile.dat" };
#endif


/** Hauptprogramm.
    @return 0 bei Erfolg, alle anderen Werte bedeuten Fehler.
*/
int
main(void)
{
    RECHTECK   re;
    FILE      *inf    = NULL;         /* Datei-Zeiger */
    int        exc    = EXIT_FAILURE; /* Exit-Code des Programmes. */
    size_t     res;                   /* Ergebnis von fread() */

    inf = hsm_et_file_open(filename, "rb", NULL);
    if (NULL != inf) {

        /* Erfolg im Rueckgabecode vermerken */
        exc = EXIT_SUCCESS;

        /* Versuchen, einen RECHTECK-Datensatz einzulesen */
        res = fread(&re, sizeof(RECHTECK), 1, inf);

        /* Wenn res = 1 ist, waren wir erfolgreich */
        if (0 < res) {
            printf(
                "Erfolg. Rechteck gelesen.\nx1=%lg\ny1=%lg\nx2=%lg\ny2=%lg\n",
                re.lu.x, re.lu.y, re.ro.x, re.ro.y
            );
        }
        else {
            fputs("FEHLER: Kein Rechteck gelesen!\n", stderr);
            fflush(stderr);
            exc = EXIT_FAILURE;
        }

        /* Falls Schliessen der Datei fehlschlaegt, Misserfolg */
        if (0 >= hsm_et_file_close(inf, filename, NULL)) {
            exc = EXIT_FAILURE;
        }
        inf = NULL;
    }
    exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 expandtab : */
