/** @file   ex037.c Demo-Programm fuer Abfrage von Datum und Uhrzeit
                    auf POSIX-Systemen.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/** Hauptprogramm.
    @return 0 bei Erfolg, alle anderen Werte weisen auf Fehler hin.
*/
int
main(void)
{
    struct tm  buffer;                    /* Aktuelle Zeit zerlegt in Teile */
    struct tm *tptr;                      /* Ergebnis von localtime_r */
    time_t     timer;                     /* Aktuelle Zeit als ein Wert */
    int        exc    =   EXIT_FAILURE;   /* Exit-Status-Code */
    
    /* Aktuelle Zeit ermitteln */
    if ((time_t)(-1) == time(&timer)) {
        /* !!! Holen der Uhrzeit ist fehlgeschlagen */
        goto finished;
    }

    /* Zeit in Komponenten zerlegen */
    tptr = localtime_r(&timer, &buffer);
    if (NULL == tptr) {
        /* !!! Fehler aufgetreten beim Zerlegen der Zeit */
        goto finished;
    }

    /* Datum und Uhrzeit ausgeben */
    printf(
        "Heute ist der %02d.%02d.%04d.\n",
        tptr->tm_mday,
        (1 + tptr->tm_mon),
        (1900 + tptr->tm_year)
    );
    printf(
        "Es ist        %02d:%02d:%02d Uhr.\n",
        tptr->tm_hour,
        tptr->tm_min,
        tptr->tm_sec
    );

    /* Erfolg vermerken */
    exc = EXIT_SUCCESS;

finished:
    exit(exc); return exc;
}

/* vim: set ai sw=4 ts=4 expandtab : */
