/** @file ex067.c Beispiel fuer Durchsuchen eines Verzeichnisses.
    Das Programm gibt die im Verzeichnis C:\Temp enthaltenen Eintraege
    aus.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <process.h>

/** Suchmuster: Verzeichnisname mit angehangenem Backslash und
    Asterisk (Stern).
    Im Beispiel wird das Verzeichnis C:\Temp durchsucht.
*/
static const char suchmuster[] = { "C:\\Temp\\*" };


/** Hauptprogramm.
    @param  argc    Anzahl der Kommandozeilenargumente.
    @param  argv    Feld mit Kommandozeilenargumenten.
                    Die Kommandozeilenargumente werden hier nicht
                    verarbeitet.
    @return 0 bei Erfolg, alle anderen Werte zeigen Fehler an.
*/
int
main(int argc, char *argv[])
{
    struct __finddata64_t fileinfo;
    intptr_t              searchid;
    int                   result;
    int                   exval = EXIT_FAILURE;

    /* Verzeichnis oeffnen, Test auf Erfolg */
    errno = 0;
    searchid = _findfirst64( suchmuster, &fileinfo );
    if (-1 != searchid) {

        /* Verzeichnis erfolgreich geoeffnet, Erfolg vermerken */
        exval = EXIT_SUCCESS;

        /* Alle Eintraege des Verzeichnisses durchlaufen */
        do {

            /* Letzten gefundenen Eintrag verarbeiten */
            fputs("Gefundener Eintrag: ", stdout);
            fputs(fileinfo.name, stdout);
            fputc('\n', stdout);

            /* Naechsten Eintrag suchen */
            errno = 0;
            result = _findnext64(searchid, &fileinfo);
            if (0 != result) {
                switch (errno) {
                    case EINVAL: {
                        fputs(
                            "FEHLER: Betriebssystem meldete Problem!\n", stderr
                        );
                        exval = EXIT_FAILURE;
                    } break;
                    case ENOENT: {
                        /* Durchsuchen beendet */
                    } break;
                    case ENOMEM: {
                        fputs(
                        "FEHLER: Zu wenig Arbeitsspeicher oder Name zu lang!\n",
                        stderr
                        );
                        exval = EXIT_FAILURE;
                    } break;
                    default: {
                        fputs(
                            "FEHLER: Ein unbekannter Fehler trat auf!\n",
                            stderr
                        );
                        exval = EXIT_FAILURE;
                    } break;
                }
            }
        } while(0 == result);

        /* Verzeichnis schliessen, Durchsuchen beenden, Ressourcen freigeben */
        _findclose(searchid);
    
    } else {

        /* Oeffnen des Verzeichnisses fehlgeschlagen, Fehlermeldung ausgeben */
        switch (errno) {
            case EINVAL: {
                fputs("FEHLER: Ungueltiger Dateiname...!\n", stderr);
            } break;
            case ENOENT: {
                fputs("FEHLER: Keine passende Datei gefunden!\n", stderr);
            } break;
            case ENOMEM: {
                fputs("FEHLER: Zu wenig Arbeitsspeicher!\n", stderr);
            } break;
            default: {
                fputs("FEHLER: Ein unbekannter Fehler trat auf!\n", stderr);
            } break;
        }

    }
    exit(exval); return exval;
}

/* vim: set ai sw=4 ts=4 expandtab : */
