/**    @file    ex080.c    Beispiel fuer Sortierung mit qsort und Suche mit bsearch.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


/** Zuordnung von Monatsnummer zum Namen.
*/
typedef struct {
    int            nr;      /**< Nummer des Monats. */
    const char    *name;    /**< Name des Monats. */
} monat_t;


/** Feld, das Angaben zu allen Monaten enthaelt.
*/
static monat_t monate[] = {
    {  1, "Januar" },     {  2, "Februar" },     {  3, "Maerz" },
    {  4, "April" },      {  5, "Mai" },         {  6, "Juni" },
    {  7, "Juli" },       {  8, "August" },      {  9, "September" },
    { 10, "Oktober" },    { 11, "November" },    { 12, "Dezember" },
};


/** Anzahl der Elemente im Feld monate.
*/
static const size_t    sz_monate = sizeof(monate)/sizeof(monat_t);


/** Vergleichsfunktion, die zwei monat_t-Datensaetze nach Namen vergleicht.
    @param    kptr    Schluesselelement bzw linkes Element.
    @param    eptr    Rechtes Element.
    @return    <0 wenn kptr<eptr, 0 wenn kptr=eptr, >0 wenn kptr>eptr.
*/
static
int
vergleiche_nach_namen(const void *kptr, const void *eptr)
{
    const monat_t    *k;
    const monat_t    *e;

    k = (const monat_t *)kptr;
    e = (const monat_t *)eptr;
    return (strcmp(k->name, e->name));
}


/** Feld monate ausgeben.
*/
static
void
feld_ausgeben(void)
{
    size_t    i;
    for (i = 0; i < sz_monate; i++) {
        printf("%02d %s\n", monate[i].nr, monate[i].name);
    }
}


/** Hauptprogramm.
    @return    EXIT_SUCCESS bei Erfolg, EXIT_FAILURE bei Fehlschlag.
*/
int
main(void)
{
    monat_t     suchschluessel;
    monat_t    *suchergebnis;

    /*  Originalzustand ausgeben.
    */
    printf("Feld monate im Originalzustand:\n");
    feld_ausgeben();

    /*  Feld monate umsortieren, so dass nach Namen geordnet.
    */
    qsort(monate, sz_monate, sizeof(monat_t), vergleiche_nach_namen);

    /*  Umsortiertes Feld ausgeben.
    */
    printf("\nFeld monate nach Umsortierung:\n");
    feld_ausgeben();

    /*  Suche nach Monat "Oktober"
        Im Element suchschluessel wird nur der Name gesetzt, da die
        Vergleichsfunktion vergleiche_nach_namen nur den Namen nutzt,
        nicht aber die Nummer.
        Bevor mit bsearch in einem Feld gesucht werden kann, muss es mit
        qsort unter Verwendung derselben Vergleichsfunktion sortiert worden
        sein. Dies ist weiter oben bereits erfolgt.
    */
    suchschluessel.name = "Oktober";
    suchergebnis = bsearch (
        &suchschluessel,
        monate, sz_monate, sizeof(monat_t), vergleiche_nach_namen
    );
    if (NULL != suchergebnis) {
        printf("\nGefunden: %02d %s\n", suchergebnis->nr, suchergebnis->name);
    }
    else {
        printf("\nFEHLER: Monat \"Oktober\" wurde nicht gefunden!\n");
    }

    /*  Suche nach nicht existierendem Monat "Urlaub".
    */
    suchschluessel.name = "Urlaub";
    suchergebnis = bsearch (
        &suchschluessel,
        monate, sz_monate, sizeof(monat_t), vergleiche_nach_namen
    );
    if (NULL != suchergebnis) {
        printf("\nGefunden: %02d %s\n", suchergebnis->nr, suchergebnis->name);
    }
    else {
        printf("\nFEHLER: Monat \"Urlaub\" wurde nicht gefunden!\n");
    }

    return(EXIT_SUCCESS);
}

/* vim: set ai sw=4 ts=4 : */
