/**	@file	he-matrix.h	Umgang mit Matrizen.

	Hier werden nur die Prototypen angegeben. Das eigentliche Modul
	mit der Implementierung der Funktionen ist durch die Studierenden
	in der Uebung zu erstellen.
*/

#ifndef HE_MATRIX_H_INCLUDED
#define	HE_MATRIX_H_INCLUDED	1

/**	Matrix mit double-Werten.
	Im eindimensionalen Datenfeld stehen zuerst alle Werte fuer
	Zeile 1, darauf folgen alle Werte fuer Zeile 2, ...
*/
typedef struct {
  double	*daten;		/**< Werte: Zeile 1, Zeile 2, usw. */
  size_t	 zeilen;	/**< Zeilenanzahl. */
  size_t	 spalten;	/**< Spaltenanzahl. */
} HE_MATRIX;

#ifdef __cplusplus
extern "C" {
#endif

/**	Neue Matrix erstellen, alle Elemente mit Anfangswert fuellen.
	@param	zeilen	Zeilenanzahl der Matrix.
	@param	spalten	Spaltenanzahl der Matrix.
	@param	wert	Initialwert fuer alle Elemente.
	@return	Zeiger auf neue Matrix bei Erfolg, NULL bei Fehler.
	Die so angelegte Matrix muss mit hsm_et_matrix_delete()
	wieder freigegeben werden.
*/
HE_MATRIX *
hsm_et_matrix_new(size_t zeilen, size_t spalten, double wert);

/**	Neue Matrix erstellen, Daten hierfuer aus Textdatei lesen.
	@param	fn	Dateiname der Textdatei.
	@return	Zeiger auf neue Matrix bei Erfolg, NULL bei Fehler.
	Die so angelegte Matrix muss mit hsm_et_matrix_delete()
	wieder freigegeben werden.
*/
HE_MATRIX *
hsm_et_matrix_from_text(const char *fn);

/**	Von der Matrix belegten Speicherplatz wieder freigeben.
	@param	pm	Zeiger auf die zu zerstoerende Matrix.
*/
void
hsm_et_matrix_delete(HE_MATRIX *pm);

/**	Matrix in Textdatei schreiben.
	@param	fn	Name der zu schreibenden Datei.
	@param	pm	Zeiger auf die zu schreibende Matrix.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
hsm_et_matrix_to_text(const char *fn, const HE_MATRIX *pm);

/**	Wert setzen.
	@param	pm	Zeiger auf zu bearbeitende Matrix.
	@param	zeile	Zeilennummer fuer Positionsangabe.
	@param	spalte	Spaltennummer fuer Positionsangabe.
	@param	wert	Der an die Position einzutragende Wert.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
hsm_et_matrix_set(HE_MATRIX *pm, size_t zeile, size_t spalte, double wert);

/**	Wert ermitteln.
	@param	pm	Zeiger auf Matrix.
	@param	zeile	Zeilennummer fuer Positionsangabe.
	@param	spalte	Spaltennummer fuer Positionsangabe.
	@return	Wert an der angegebenen Position bei Erfolg, 0 bei Fehler.
*/
double
hsm_et_matrix_get(const HE_MATRIX *pm, size_t zeile, size_t spalte);

/**	Maximum-Suche.
	@param	pm	Zeiger auf die zu untersuchende Matrix.
	@return	Maximalwert der Matrixelemente.
*/
double
hsm_et_matrix_max(const HE_MATRIX *pm);

/**	Minimum-Suche.
	@param	pm	Zeiger auf die zu untersuchende Matrix.
	@return	Minimum der Werte.
*/
double
hsm_et_matrix_min(const HE_MATRIX *pm);

/**	Summenberechnung.
	@param	pm	Zeiger auf die zu untersuchende Matrix.
	@return	Summe aller Elemente der Matrix.
*/
double
hsm_et_matrix_sum(const HE_MATRIX *pm);

/**	Mittelwertberechnung.
	@param	pm	Zeiger auf die zu untersuchende Matrix.
	@return	Mittelwert aller Matrixelemente.
*/
double
hsm_et_matrix_avg(const HE_MATRIX *pm);

/**	Matrizenmultiplikation.
	Die beiden Matrizen a und b werden miteinander multipliziert,
	das Ergebnis in res abgespeichert.
	@param	r	Zeiger auf Ergebnismatrix.
	@param	a	Linke Matrix fuer Multiplikation.
	@param	b	Rechte Matrix fuer Multiplikation.
	@return	1 fuer Erfolg, 0 bei Fehler.
*/
int
hsm_et_matrix_mult(HE_MATRIX *r,const HE_MATRIX *a,const HE_MATRIX *b);

#ifdef __cplusplus
}
#endif

#endif
/* ifndef HE_MATRIX_H_INCLUDED */

/* vim: set ai sw=4 ts=4 expandtab : */

