/** @file   he-read.c   Daten von Standardeingabe einlesen,
    bei Problemen Fehlermeldung ausgeben.
*/

/* Copyright (C) 2014 - HS Schmalkalden. All rights reserved. */

#ifdef _WIN32
#define	_CRT_SECURE_NO_WARNINGS 1
#endif

#include "he-conf.h"

#include <stdio.h>
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_LIMITS_H
#include <limits.h>
#endif
#if HAVE_STDINT_H
#include <stdint.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif

#include "he-read.h"
#include "he-strch.h"



/**	Texte fuer Fehlermeldungen.
*/
static const char * const	hsm_et_read_texts[] = {
/*  0 */
"FEHLER: Keine Eingabezeile lesbar!\n",

/*  1 */
"FEHLER: Abbruch wegen Eingabe einer Leerzeile!\n",

/*  2 */
"FEHLER: Text zu lang!\n",

/*  3, 4 */
"FEHLER: Kein Ganzzahl-Wert: \"",
"\"!\n",

/*  5 */
"FEHLER: Zahlenbereichsueberlauf!\n",

/*  6 */
"Warnung: Nicht auswertbarer Text am Ende der Eingabe: \""
};



int
hsm_et_read_double(char const *msg, double *dptr)
{
  char		 buffer[128];
  char		*ptr;
  char		*eptr;
  double	 dres;
  int		 res = 0;

  if (NULL != dptr) {
    if (NULL != msg) {
      fputs(msg, stdout);
      fflush(stdout);
    }
    if (fgets(buffer, sizeof(buffer), stdin)) {
      ptr = hsm_et_str_chomp(buffer);
      if (NULL != ptr) {
        eptr = NULL;
	dres = strtod(ptr, &eptr);
	if (eptr != ptr) {
	  *dptr = dres;
	  res = 1;
	  if ('\0' != *eptr) {
	    /* WARNUNG: Unverarbeitbare Zeichen am Ende */
            fputs(hsm_et_read_texts[6], stderr);
	    fputs(eptr, stderr);
	    fputs(hsm_et_read_texts[4], stderr);
            fflush(stderr);
	  }
	} else {
	  /* FEHLER Text nicht verarbeitbar */
          fputs(hsm_et_read_texts[3], stderr);
	  fputs(ptr, stderr);
	  fputs(hsm_et_read_texts[4], stderr);
          fflush(stderr);
	}
      } else {
        /* FEHLER: Leerzeile eingegeben */
        fputs(hsm_et_read_texts[1], stderr);
        fflush(stderr);
      }
    } else {
      /* FEHLER: Eingabezeile konnte nicht gelesen werden */
      fputs(hsm_et_read_texts[0], stderr);
      fflush(stderr);
    }
  }
  return res;
}



int
hsm_et_read_int(char const *msg, int *dptr)
{
  char		 buffer[64];
  char		*ptr;
  char		*eptr;
  long		 lres;
  int		 res = 0;
  if (NULL != dptr) {
    if (NULL != msg) {
      fputs(msg, stdout);
      fflush(stdout);
    }
    if (fgets(buffer, sizeof(buffer), stdin)) {
      ptr = hsm_et_str_chomp(buffer);
      if (NULL != ptr) {
        eptr = NULL;
	lres = strtol(ptr, &eptr, 10);
	if (eptr != ptr) {
	  if (((long)(INT_MIN) <= lres) && ((long)(INT_MAX) >= lres)) {
	    *dptr = (int)lres;
	    res = 1;
	    if ('\0' != *eptr) {
	      /* WARNUNG: Nicht auswertbare Zeichen */
              fputs(hsm_et_read_texts[6], stderr);
	      fputs(eptr, stderr);
	      fputs(hsm_et_read_texts[4], stderr);
              fflush(stderr);
	    }
	  } else {
	    /* FEHLER Unter- bzw. Ueberlauf */
            fputs(hsm_et_read_texts[5], stderr);
            fflush(stderr);
	  }
	} else {
	  /* FEHLER Text nicht verarbeitbar */
          fputs(hsm_et_read_texts[3], stderr);
	  fputs(ptr, stderr);
	  fputs(hsm_et_read_texts[4], stderr);
          fflush(stderr);
	}
      } else {
        /* FEHLER: Leerzeile eingegeben */
        fputs(hsm_et_read_texts[1], stderr);
        fflush(stderr);
      }
    } else {
      /* FEHLER: Eingabezeile konnte nicht gelesen werden */
      fputs(hsm_et_read_texts[0], stderr);
      fflush(stderr);
    }
  }
  return res;
}



int
hsm_et_read_string(char const *msg, char *dptr, char *buffer, size_t sz)
{
  char		*ptr;
  int		 res	= 0;
  if ((NULL != dptr) && (NULL != buffer) && (0 < sz)) {
    if (NULL != msg) {
      fputs(msg, stdout);
      fflush(stdout);
    }
    if (fgets(buffer, sz, stdin)) {
      ptr = hsm_et_str_chomp(buffer);
      if (NULL != ptr) {
        if (sz > strlen(ptr)) {
	  /* Erfolg */
	  strcpy(dptr, ptr);
	  res = 1;
	} else {
	  /* BUG: Puffer zu klein */
          fputs(hsm_et_read_texts[2], stderr);
          fflush(stderr);
	}
      } else {
        /* FEHLER: Leerzeile */
        fputs(hsm_et_read_texts[1], stderr);
        fflush(stderr);
      }
    } else {
      /* FEHLER: Zeile konnte nicht gelesen werden */
      fputs(hsm_et_read_texts[0], stderr);
      fflush(stderr);
    }
  }
  return res;
}



/* vim: set ai sw=4 ts=4 expandtab : */
