/**	@file	he-ulong.h	Mathematische Operationen
	mit unsigned-long-Werten, Erkennung von Ueberlauf,
	Vorbeugung vor Division durch 0.

*/

/* Copyright (C) 2014 - HS Schmalkalden. All rights reserved. */

#ifndef HE_ULONG_H_INCLUDED
/** Schutz vor mehrfacher Inklusion */
#define	HE_ULONG_H_INCLUDED 1


#ifdef __cplusplus
extern "C" {
#endif

/** Addition.
    @param	a	Linker Operand.
    @param	b	Rechter Operand.
    @param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    @return	Ergebnis.
*/
unsigned long
hsm_et_ulong_add(unsigned long a, unsigned long b, int *ec);

/** Subtraktion.
    @param	a	Linker Operand.
    @param	b	Rechter Operand.
    @param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    @return	Ergebnis.
*/
unsigned long
hsm_et_ulong_sub(unsigned long a, unsigned long b, int *ec);

/** Multiplikation.
    @param	a	Linker Operand.
    @param	b	Rechter Operand.
    @param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    @return	Ergebnis.
*/
unsigned long
hsm_et_ulong_mul(unsigned long a, unsigned long b, int *ec);

/** Division.
    @param	a	Zaehler.
    @param	b	Nenner.
    @param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    @return	Ergebnis.
*/
unsigned long
hsm_et_ulong_div(unsigned long a, unsigned long b, int *ec);

/** Quadrieren.
    @param	a	Linker Operand.
    @param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
    @return	Ergebnis.
*/
unsigned long
hsm_et_ulong_sqr(unsigned long a, int *ec);

#ifdef __cplusplus
}
#endif

#endif
/* ifndef HE_ULONG_H_INCLUDED */

/* vim: set ai sw=4 ts=4 expandtab : */

