/* Flaeche eines achsparallelen Rechteckes berechnen.
   Dieses Beispiel demonstriert den Zugriff auf struct-Komponenten
   ueber Zeiger.
*/

/* Copyright (C) 2014-2018 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>
#include <math.h>


/** Koordinaten fuer einen Punkt.
*/
typedef struct {
	double    x;  /**< x-Koordinate. */
	double    y;  /**< y-Koordinate. */
} PUNKT;


/** Achsenparalleles Rechteck.
*/
typedef struct {
	PUNKT lu; /**< Eckpunkt links unten. */
	PUNKT ro; /**< Eckpunkt rechts oben. */
} RECHTECK;


/** Rechteck-Daten.
*/
const RECHTECK re = { { 1.0, 2.0 }, { 3.0, 4.0} };


/** Hauptprogramm.
	@return 0 bei Erfolg, alle anderen Werte bedeuten Fehler.
*/
int
main(void)
{
	const RECHTECK    *reptr;
	double             breite;
	double             hoehe;
	double             flaeche;

	/*	Zugriff ueber Zeiger.
		Der Zeiger reptr wird auf die Anfangsadresse des Rechteckes
		re gesetzt.
		reptr->ro liefert die Komponente ro des Rechteckes, auf das
		der Zeiger reptr zeigt. Das Resultat is ein PUNKT.
		Das angehangene .x selektiert hiervon die Komponente x. Das
		Resultat ist ein double-Wert (die x-Koordinate des Punktes).
	*/
	reptr  = &re;
	breite = reptr->ro.x - reptr->lu.x;
	hoehe  = reptr->ro.y - reptr->lu.y;
	/*
		Alternativ kann auch mit dem Operator * der Zeiger
		dereferenziert werden, dann kann mit dem Operator .
		auf die Komponenten zugegriffen werden.
	*/
	hoehe  = (*reptr).ro.y - (*reptr).lu.y;
	/* fabs() bildet Betrag. */
	flaeche = fabs(breite * hoehe);
	printf("Flaeche: %lg\n", flaeche);
	return 0;
}

/* vim: set ai sw=4 ts=4 : */
