/** @file   ex068.c Beispiel fuer Directory-Durchsuchen unter Linux/UNIX.
	Das Programm gibt die Namen der Eintraege im aktuellen Verzeichnis aus.
*/

/* Copyright (C) 2014-2017 - HS Schmalkalden. All rights reserved. */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>


int
main(int argc, char *argv[])
{
	DIR           *pdir;                      /* Directory */
	struct dirent *pent;                      /* Directory-Eintag */
	int            exval = EXIT_FAILURE;      /* Exit-Status-Code */

	/* Verzeichnis oeffnen, auf Erfolg testen */
	errno = 0;
	pdir = opendir(".");
	if (NULL != pdir) {       /* Verzeichnis erfolgreich geoeffnet */

		/* Verzeichnis durchsuchen */
		while ( NULL   !=   (pent = readdir(pdir)) ) {
			fputs("Gefundener Eintrag: ", stdout);
			fputs(pent->d_name, stdout);
			fputc('\n', stdout);
		}

		/* Verzeichnis wieder schliessen */
		closedir(pdir);

	}
	else {                  /* Fehler beim Verzeichnis-Oeffnen */

		switch (errno) {
			case EACCES: {
				fputs("FEHLER: Keine Berechtigung!\n", stderr);
			} break;
			case EMFILE: {
				fputs(
					"FEHLER: Prozess hat zu viele Dateien geoeffnet!\n", stderr
				);
			} break;
			case ENFILE: {
				fputs(
					"FEHLER: Zu viele Dateien sind im System geoeffnet!\n",
					stderr
				);
			} break;
			case ENOENT: {
				fputs("FEHLER: Verzeichnis existiert nicht!\n", stderr);
			} break;
			case ENOMEM: {
				fputs("FEHLER: Zu wenig Speicher verfuegbar!\n", stderr);
			} break;
			case ENOTDIR: {
				fputs("FEHLER: Der Dateiname ist kein Verzeichnis!\n", stderr);
			} break;
			default: {
				fputs("FEHLER: Unbekanntes Problem aufgetreten!\n", stderr);
			} break;
		}
	
	}
	exit(exval); return exval;
}

/* vim: set ai sw=4 ts=4 : */
