/** @file   he-bfio.h   Lesen und Schreiben von Binaerdateien.

Für ganzzahlige Werte wird das niederwertigste Byte zuerst in der Datei
gespeichert.

Fuer Strings (Zeichenketten) wird zuerst die Laenge als vorzeichenloser
16-Bit-Wert gespeichert, daran schliesst sich der Text selbst an.
Das abschliessende Null-Byte wird sowohl in der Laengenangabe mit gezaehlt
als auch in der Datei gespeichert.
*/

#ifndef	HE_BFIO_H_INCLUDED

/** Schutz vor Mehrfach-include. */
#define	HE_BFIO_H_INCLUDED	1


#include <stdio.h>


#ifdef  __cplusplus
extern "C" {
#endif

/**	Vorzeichenlose 16-Bit-Zahl binaer in Datei speichern.
	@param	fp	Binaerdatei, zum Schreiben geoeffnet.
	@param	u16	Zu speichernde Zahl.
	@param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_save_uint16(FILE *fp, unsigned short u16, int *ec);

/**	Vorzeichenlose 32-Bit-Zahl binaer in Datei speichern.
	@param	fp	Binaerdatei, zum Schreiben geoeffnet.
	@param	u32	Zu speichernde Zahl.
	@param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_save_uint32(FILE *fp, unsigned long u32, int *ec);

/**	Vorzeichenbehaftete 16-Bit-Zahl binaer in Datei speichern.
	@param	fp	Binaerdatei, zum Schreiben geoeffnet.
	@param	i16	Zu speichernde Zahl.
	@param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_save_int16(FILE *fp, short i16, int *ec);

/**	Vorzeichenbehaftete 32-Bit-Zahl binaer in Datei speichern.
	@param	fp	Binaerdatei, zum Schreiben geoeffnet.
	@param	i32	Zu speichernde Zahl.
	@param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_save_int32(FILE *fp, long i32, int *ec);

/**	String binaer in Datei speichern.
	@param	fp	Binaerdatei, zum Schreiben geoeffnet.
	@param	str	Zu speichernder String.
	@param	ec	Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_save_string(FILE *fp, const char *str, int *ec);

/**	Vorzeichenlose 16-Bit-Zahl binaer aus Datei lesen.
	@param	u16p	Adresse der Zielvariable.
	@param	fp		Binaerdatei, zum Lesen geoeffnet.
	@param	ec		Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_restore_uint16(unsigned short *u16p, FILE *fp, int *ec);

/**	Vorzeichenlose 32-Bit-Zahl binaer aus Datei lesen.
	@param	u32p	Adresse der Zielvariable.
	@param	fp		Binaerdatei, zum Lesen geoeffnet.
	@param	ec		Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_restore_uint32(unsigned long *u32p, FILE *fp, int *ec);

/**	Vorzeichenbehaftete 16-Bit-Zahl binaer aus Datei lesen.
	@param	i16p	Adresse der Zielvariable.
	@param	fp		Binaerdatei, zum Lesen geoeffnet.
	@param	ec		Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_restore_int16(short *i16p, FILE *fp, int *ec);

/**	Vorzeichenbehaftete 32-Bit-Zahl binaer aus Datei lesen.
	@param	i32p	Adresse der Zielvariable.
	@param	fp		Binaerdatei, zum Lesen geoeffnet.
	@param	ec		Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_restore_int32(long *i32p, FILE *fp, int *ec);

/**	String binaer aus Datei lesen.
	@param	strp	Adresse der Zeigervariable.
	@param	fp		Binaerdatei, zum Lesen geoeffnet.
	@param	ec		Zeiger auf Fehlercode-Variable, darf NULL sein.
	@return	1 bei Erfolg, 0 bei Fehler.
*/
int
he_bfio_restore_string(char **strp, FILE *fp, int *ec);


#ifdef  __cplusplus
}
#endif

#endif
/* ifndef HE_BFIO_H_INCLUDED */


/* vim: set ai sw=4 ts=4 : */

